
@import url("basic.css");

html, body{
	font-family:Arial,sans-serif;
	margin: 0px;
	padding: 0px;
	background-color: #fff;
	color: #222;
	line-height: 150%;
	font-size: 11pt; 
}
code, pre {
	font: 1em/normal "courier new", courier, monospace;
}
h1, h2, h3, h4 {
  color: #8b0000;
}
a {
	color: #8b0000;
	text-decoration: none;
}
#header {
	xmargin: 3em -2em 1em 0;
	padding: 1.2em 0 1.2em 3em;
	border-width: 0px;
	background-color: #8b0000;
	/*background-color: #369;*/
	xwidth: 80%;
	min-width: 900px;
}
.title {
	color: white;
	font-size: 24pt;
	margin: 2px;
}
.subtitle, .projects, .projects a {
	color: white;
	font-size: 14pt;
	font-style: italic;
	margin: 2px;
	line-height: 1.5;
}
.projects, .projects a {
	font-style: normal;
	font-size: 11pt;
}
.lsidebar {
	float: left;
	font-size: 12pt;
	color: #8b0000;
	width: 150px;
}
.lsidebar li {
	list-style-type: none;
}
.lsidebar li a {
	text-decoration: none;
	color: #8b0000;
}
.lsidebar li a:hover {
	text-decoration: underline;
	color: #F33;
}
.lsidebar li a:focus {
	text-decoration: underline;
	color: #F33;
}
.lsidebar li a:active {
	text-decoration: underline;
	color: #F33;
}
.lsidebar li.missing {
        text-decoration: none;
	color: #8b8888;
}
.content {
	/*width: 700px;*/
	clear: none;
        margin-left: 180px;
	margin-right: 20px;
	xpadding: 10px;
	min-height: 800px;
}
.numeric {
  text-align: right;
}
.data-table {
  border: 1px solid #a9a9a9;
  border-collapse: collapse;
}
.data-table td, .data-table th {
  border: 1px solid #888;
  padding: 2px;
}
.data-table th {
  background-color: #ccc;
  font-weight: bold;
}
.advert-message {
  border: 3px solid #8b0000;
  padding: 15px;
}
a:hover {
	color: #F33;
	text-decoration: underline;
}
#footer {
	/*xmargin: 3em -2em 1em 0;*/
	padding: 0.1em 0 0.1em 4em;
	border-width: 0px;
	/*background-color: #8b0000;*/
	/*background-color: #369;*/
/*	xwidth: 80%;*/
	min-width: 900px;
} 
#footer .projects {
	font-style: normal;
	font-size: 10pt;
	color: white;
}
#header a:hover, #footer a:hover {
	color: #F66;
	text-decoration: underline;
}
#footer .projects , #footer  a {
	font-style: normal;
	font-size: 10pt;
	color: #8b0000;
}

div.sphinxsidebar {
    float: right;
    margin-left: 0%;
}